#!/usr/bin/env python
# -*- coding: UTF-8 -*-


from fablinker.exceptions import ConfigParseError
from fablinker.fabshell import FabShell
from fablinker.utils import ColorPrint, get_cmd_args
from fablinker.constant import __version__
import traceback


def print_welcome():
    wel = '''
            **************************************************************
            Welcome to fablinker V%s
            Type help for more information.
            Enjoy your operations! (*^_^*)
            **************************************************************
            ''' % __version__
    ColorPrint.blue(wel)


def main():

    try:
        conf_file = get_cmd_args()

        fabshell = FabShell(conf_file)
        print_welcome()
        fabshell.cmdloop()
    except ConfigParseError as e:
        exstr = traceback.format_exc()
        ColorPrint.red(exstr)

    except Exception as e:
        ColorPrint.red(e.message)


#!/usr/bin/env python
# -*- coding: UTF-8 -*-

import sys
import shlex
import os
import platform
import zipfile
import six

# If you ever want to import from the vendored six. Add it here and then
# import from hcmd.compat. Also try to keep it in alphabetical order.
# This may get large.
advance_iterator = six.advance_iterator
PY3 = six.PY3
queue = six.moves.queue
shlex_quote = six.moves.shlex_quote
StringIO = six.StringIO
BytesIO = six.BytesIO
urlopen = six.moves.urllib.request.urlopen
binary_type = six.binary_type

# Most, but not all, python installations will have zlib. This is required to
# compress any files we send via a push. If we can't compress, we can still
# package the files in a zip container.
try:
    import zlib
    ZIP_COMPRESSION_MODE = zipfile.ZIP_DEFLATED
except ImportError:
    ZIP_COMPRESSION_MODE = zipfile.ZIP_STORED


try:
    import sqlite3
except ImportError:
    sqlite3 = None

is_windows = sys.platform == 'win32'

if is_windows:
    from hcmd.multithreading import *
else:
    import multiprocessing
    from multiprocessing import Process, Lock, Value
    List = multiprocessing.Manager().list
    Dict = multiprocessing.Manager().dict



if is_windows:
    default_pager = 'more'
else:
    default_pager = 'less -R'


class StdinMissingError(Exception):
    def __init__(self):
        message = (
            'stdin is required for this operation, but is not available.'
        )
        super(StdinMissingError, self).__init__(message)


class NonTranslatedStdout(object):
    """ This context manager sets the line-end translation mode for stdout.

    It is deliberately set to binary mode so that `\r` does not get added to
    the line ending. This can be useful when printing commands where a
    windows style line ending would casuse errors.
    """

    def __enter__(self):
        if sys.platform == "win32":
            import msvcrt
            self.previous_mode = msvcrt.setmode(sys.stdout.fileno(),
                                                os.O_BINARY)
        return sys.stdout

    def __exit__(self, type, value, traceback):
        if sys.platform == "win32":
            import msvcrt
            msvcrt.setmode(sys.stdout.fileno(), self.previous_mode)


def ensure_text_type(s):
    if isinstance(s, six.text_type):
        return s
    if isinstance(s, six.binary_type):
        return s.decode('utf-8')
    raise ValueError("Expected str, unicode or bytes, received %s." % type(s))


def safe_encode(item):
    if not six.PY2:
        return item
    if isinstance(item, six.string_types):
        try:
            item = item.encode('UTF-8') if not is_windows else item.encode('gbk')
        except UnicodeDecodeError:
            try:
                item = item.encode('GB2312')
            except Exception:
                item = None
    return item


def safe_encode_file(item):
    if not six.PY2:
        return item
    if isinstance(item, six.string_types):
        try:
            # item = item.encode('UTF-8') if not is_windows else item.encode('gbk')
            item = item.encode('UTF-8')
        except UnicodeDecodeError:
            try:
                item = item.encode('GB2312')
            except Exception:
                item = None
    return item


def safe_decode(item):
    if not six.PY2:
        return item
    if isinstance(item, str):
        try:
            item = item.decode('UTF-8')
        except UnicodeDecodeError:
            try:
                item = item.decode('GB2312')
            except Exception:
                item = None
    return item


if six.PY3:
    import locale
    import urllib.parse as urlparse

    from urllib.error import URLError

    raw_input = input

    def get_binary_stdin():
        if sys.stdin is None:
            raise StdinMissingError()
        return sys.stdin.buffer

    def get_binary_stdout():
        return sys.stdout.buffer

    def _get_text_writer(stream, errors):
        return stream

    def compat_open(filename, mode='r', encoding=None):
        """Back-port open() that accepts an encoding argument.

        In python3 this uses the built in open() and in python2 this
        uses the io.open() function.

        If the file is not being opened in binary mode, then we'll
        use locale.getpreferredencoding() to find the preferred
        encoding.

        """
        if 'b' not in mode:
            encoding = locale.getpreferredencoding()
        return open(filename, mode, encoding=encoding)

    def bytes_print(statement, stdout=None):
        """
        This function is used to write raw bytes to stdout.
        """
        if stdout is None:
            stdout = sys.stdout

        if getattr(stdout, 'buffer', None):
            stdout.buffer.write(statement)
        else:
            # If it is not possible to write to the standard out buffer.
            # The next best option is to decode and write to standard out.
            stdout.write(statement.decode('utf-8'))

else:
    import codecs
    import locale
    import io
    import urlparse

    from urllib2 import URLError

    raw_input = raw_input

    def get_binary_stdin():
        if sys.stdin is None:
            raise StdinMissingError()
        return sys.stdin

    def get_binary_stdout():
        return sys.stdout

    def _get_text_writer(stream, errors):
        # In python3, all the sys.stdout/sys.stderr streams are in text
        # mode.  This means they expect unicode, and will encode the
        # unicode automatically before actually writing to stdout/stderr.
        # In python2, that's not the case.  In order to provide a consistent
        # interface, we can create a wrapper around sys.stdout that will take
        # unicode, and automatically encode it to the preferred encoding.
        # That way consumers can just call get_text_writer(stream) and write
        # unicode to the returned stream.  Note that get_text_writer
        # just returns the stream in the PY3 section above because python3
        # handles this.

        # We're going to use the preferred encoding, but in cases that there is
        # no preferred encoding we're going to fall back to assuming ASCII is
        # what we should use. This will currently break the use of
        # PYTHONIOENCODING, which would require checking stream.encoding first,
        # however, the existing behavior is to only use
        # locale.getpreferredencoding() and so in the hope of not breaking what
        # is currently working, we will continue to only use that.
        encoding = locale.getpreferredencoding()
        if encoding is None:
            encoding = "ascii"

        return codecs.getwriter(encoding)(stream, errors)

    def compat_open(filename, mode='r', encoding=None):
        # See docstring for compat_open in the PY3 section above.
        if 'b' not in mode:
            encoding = locale.getpreferredencoding()
        return io.open(filename, mode, encoding=encoding)

    def bytes_print(statement, stdout=None):
        if stdout is None:
            stdout = sys.stdout

        stdout.write(statement)


def get_stdout_text_writer():
    return _get_text_writer(sys.stdout, errors="strict")


def get_stderr_text_writer():
    return _get_text_writer(sys.stderr, errors="replace")


def compat_input(prompt):
    """
    Cygwin's pty's are based on pipes. Therefore, when it interacts with a Win32
    program (such as Win32 python), what that program sees is a pipe instead of
    a console. This is important because python buffers pipes, and so on a
    pty-based terminal, text will not necessarily appear immediately. In most
    cases, this isn't a big deal. But when we're doing an interactive prompt,
    the result is that the prompts won't display until we fill the buffer. Since
    raw_input does not flush the prompt, we need to manually write and flush it.

    See https://github.com/mintty/mintty/issues/56 for more details.
    """
    sys.stdout.write(prompt)
    sys.stdout.flush()
    return raw_input()


def compat_shell_quote(s, platform=None):
    """Return a shell-escaped version of the string *s*

    Unfortunately `shlex.quote` doesn't support Windows, so this method
    provides that functionality.
    """
    if platform is None:
        platform = sys.platform

    if platform == "win32":
        return _windows_shell_quote(s)
    else:
        return shlex_quote(s)


def _windows_shell_quote(s):
    """Return a Windows shell-escaped version of the string *s*

    Windows has potentially bizarre rules depending on where you look. When
    spawning a process via the Windows C runtime the rules are as follows:

    https://docs.microsoft.com/en-us/cpp/cpp/parsing-cpp-command-line-arguments

    To summarize the relevant bits:

    * Only space and tab are valid delimiters
    * Double quotes are the only valid quotes
    * Backslash is interpreted literally unless it is part of a chain that
      leads up to a double quote. Then the backslashes escape the backslashes,
      and if there is an odd number the final backslash escapes the quote.

    :param s: A string to escape
    :return: An escaped string
    """
    if not s:
        return '""'

    buff = []
    num_backspaces = 0
    for character in s:
        if character == '\\':
            # We can't simply append backslashes because we don't know if
            # they will need to be escaped. Instead we separately keep track
            # of how many we've seen.
            num_backspaces += 1
        elif character == '"':
            if num_backspaces > 0:
                # The backslashes are part of a chain that lead up to a
                # double quote, so they need to be escaped.
                buff.append('\\' * (num_backspaces * 2))
                num_backspaces = 0

            # The double quote also needs to be escaped. The fact that we're
            # seeing it at all means that it must have been escaped in the
            # original source.
            buff.append('\\"')
        else:
            if num_backspaces > 0:
                # The backslashes aren't part of a chain leading up to a
                # double quote, so they can be inserted directly without
                # being escaped.
                buff.append('\\' * num_backspaces)
                num_backspaces = 0
            buff.append(character)

    # There may be some leftover backspaces if they were on the trailing
    # end, so they're added back in here.
    if num_backspaces > 0:
        buff.append('\\' * num_backspaces)

    new_s = ''.join(buff)
    if ' ' in new_s or '\t' in new_s:
        # If there are any spaces or tabs then the string needs to be double
        # quoted.
        return '"%s"' % new_s
    return new_s


def get_popen_kwargs_for_pager_cmd(pager_cmd=None):
    """Returns the default pager to use dependent on platform

    :rtype: str
    :returns: A string represent the paging command to run based on the
        platform being used.
    """
    popen_kwargs = {}
    if pager_cmd is None:
        pager_cmd = default_pager
    # Similar to what we do with the help command, we need to specify
    # shell as True to make it work in the pager for Windows
    if is_windows:
        popen_kwargs = {'shell': True}
    else:
        pager_cmd = shlex.split(pager_cmd)
    popen_kwargs['args'] = pager_cmd
    return popen_kwargs


try:
    from collections import OrderedDict
except ImportError:
    # Python2.6 we use the 3rd party back port.
    from ordereddict import OrderedDict


def config_parser():
    return six.moves.configparser.ConfigParser()


#!/usr/bin/env python
# -*- coding: UTF-8 -*-
import os

# from utils import get_full_path

__version__ = '1.2.2'

CONF_FILE_NAME = 'fabconf.ini'

HCMD_DIR = os.path.expanduser(os.path.join('~', '.fablinker'))
CONFIG_FILE = os.path.join(HCMD_DIR, CONF_FILE_NAME)

CMD_PROMPT = '>>'



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
import argparse

from fabric.colors import *
from fablinker.compat import config_parser, compat_input
from fablinker.constant import CONFIG_FILE, __version__
from fablinker.exceptions import ConfigParseError, FileNotFoundError


class ColorPrint(object):
    @staticmethod
    def red(msg):
        print(red(msg))

    @staticmethod
    def blue(msg):
        print(blue(msg))

    @staticmethod
    def yellow(msg):
        print(yellow(msg))


def get_cmd_args():
    parser = argparse.ArgumentParser(description='Fablinker is a tool for operating servers!')

    parser.add_argument('-f', action='store', dest='conf_file', help='server hosts file')
    parser.add_argument("-v", "--version", help="show the version of fablinker", action="store_true")
    cmd_args = parser.parse_args()

    if cmd_args.version:
        print(__version__)
        exit(0)
    conf_file = cmd_args.conf_file if cmd_args.conf_file else CONFIG_FILE
    if not os.path.isfile(conf_file):
        raise FileNotFoundError("config file not found!")
    return conf_file


def parse_config(conf_file):
    base_conf = {}
    host_groups = {}
    try:
        cf = config_parser()
        cf.optionxform = str
        cf.read(conf_file)
        baseconf = cf.options('baseconf')
        for key in baseconf:
            base_conf[key] = cf.get('baseconf', key)
        groups = cf.options('host_groups')
        if len(groups) < 1:
            raise ConfigParseError("parse config file error, hosts not find!")
        current_group = groups[0]

        for group in groups:
            hosts = cf.get('host_groups', group)
            hosts = hosts.replace('[', '').replace(']', '').replace(' ', '').split(',')
            host_groups[group] = hosts
        return base_conf, host_groups, current_group
    except Exception as e:
        raise ConfigParseError("parse config file error, %s" % e)


def get_full_path(path):
    fullpath = os.path.expandvars(path)
    fullpath = os.path.expanduser(fullpath)
    fullpath = os.path.abspath(fullpath)
    return fullpath


def warning_prompt(prompt_text):

    msg = red("%s(yes/no): " % (prompt_text,))
    response = compat_input(msg)
    while response not in ['yes', 'no']:
        response = compat_input(msg)
    return response == 'yes'






#!/usr/bin/env python
# -*- coding: UTF-8 -*-


from __future__ import unicode_literals


class BaseError(Exception):
    """
    The base exception class for fablinker base exceptions.

    :ivar msg: The descriptive message associated with the error.
    """
    fmt = 'An unspecified error occurred'

    def __init__(self, fmt=None, **kwargs):
        if fmt is not None:
            self.fmt = fmt
        msg = self.fmt.format(**kwargs)
        Exception.__init__(self, msg)
        self.kwargs = kwargs


class ConfigParseError(BaseError):
    """
    config file error
    """
    fmt = 'Parse config file error.'


class FileNotFoundError(BaseError):
    """
    file not found
    """
    fmt = 'file not found.'





#!/usr/bin/env python
# -*- coding: UTF-8 -*-

from __future__ import print_function
import cmd
import getpass
import subprocess
import functools
import fabric.api as fab
from fabric.colors import *

from fablinker.compat import config_parser
from fablinker.constant import CMD_PROMPT
from fablinker.utils import ColorPrint, parse_config, warning_prompt


def fab_execute(func):
    @functools.wraps(func)
    def wrapper(*args):
        return fab.execute(func, *args)
    return wrapper


def need_confirm(func):
    @functools.wraps(func)
    def wrapper(*args):
        isok = warning_prompt('this is a dangerous command, sure to execute ? ')
        return fab.execute(func, *args) if isok else True
    return wrapper


class FabShell(cmd.Cmd):
    """
    A simple fabric shell
    """
    def __init__(self, conf_file):
        cmd.Cmd.__init__(self)
        self.conf_file = conf_file
        self.base_conf, self.host_groups, self.current_group = parse_config(conf_file)

        for key in self.base_conf:
            fab.env[key] = self.base_conf[key]

        fab.env.hosts = self.host_groups[self.current_group]
        if not fab.env.user:
            fab.env.user = getpass.getuser()
        fab.env.warn_only = True
        fab.env.colorize_errors = True
        fab.env.skip_bad_hosts = True

        self.cwd_list = ['home', fab.env.user]
        self.conf_changed = False
        self.all_hosts = set([])
        self.disconnect_hosts = set([])
        self.set_cmd_prompt(self.current_group)

    def do_save(self, *args):
        """
        save configure file
        """
        self.save_config(self.conf_file)

    def save_config(self, conf_file):
        """
        save configure file
        """
        try:
            cf = config_parser()
            cf.optionxform = str
            cf.read(conf_file)
            cf.set('baseconf', 'user', fab.env.user)
            cf.set('baseconf', 'password', fab.env.password)
            cf.set('baseconf', 'parallel', fab.env.parallel)
            for group, hosts in self.host_groups.items():
                hosts_str = '[' + ','.join(hosts) + ']'
                cf.set('host_groups', group, hosts_str)
            cf.write(open(conf_file, 'w'))
            return True
        except Exception as e:
            ColorPrint.red(e.message)

    def set_fabp(self, *args):
        args_list = args[0]
        fab_kw = args[1]
        args_list.remove('-fabp')
        fab_kw['parallel'] = True

    def set_fabs(self, *args):
        args_list = args[0]
        fab_kw = args[1]
        args_list.remove('-fabs')
        fab_kw['parallel'] = False

    def set_fabh(self, *args):
        args_list = args[0]
        fab_kw = args[1]

        idx = args_list.index('-fabh')
        hosts_str = args_list.pop(idx+1)
        fab_hosts = hosts_str.split(',')

        args_list.remove('-fabh')
        fab_kw['hosts'] = fab_hosts

    @fab_execute
    def fab_run(self, args):
        try:
            return fab.run(args)
        except Exception as e:
            raise e

    def fab_task_cd(self, args):
        ret = self.fab_run(args)
        for host in ret:
            if not ret[host].succeeded:
                return True
        tmp_cwd = args.split()[-1]
        if tmp_cwd.startswith('~'):
            self.cwd_list = ['home', fab.env.user]
            self.cwd_list += tmp_cwd[1:].strip('/').split('/')
        elif tmp_cwd.startswith('..'):
            self.cwd_list.pop()
            self.cwd_list += tmp_cwd[2:].strip('/').split('/')
        elif tmp_cwd.startswith('.'):
            self.cwd_list += tmp_cwd[1:].strip('/').split('/')
        elif tmp_cwd.startswith('/'):
            self.cwd_list = tmp_cwd.strip('/').split('/')
        else:
            self.cwd_list += tmp_cwd.strip('/').split('/')

        fab.env.cwd = '/' + '/'.join(self.cwd_list)
        print (fab.env.cwd)
        print ()
        return True

    def fab_task_checkhosts(self, args):
        for group, hosts in self.host_groups.items():
            self.all_hosts |= set(hosts)
        for host in self.all_hosts:
            try:
                with fab.settings(hosts=[host]):
                    with fab.hide('running', 'stdout', 'stderr'):
                        ret = self.fab_run('date')
                        if ret[host].succeeded:
                            print(host, 'connected ok.')
                        else:
                            ColorPrint.red('%s connected failed!' % host)
                            self.disconnect_hosts.add(host)
            except Exception as e:
                ColorPrint.red('%s connected failed!' % host)
                self.disconnect_hosts.add(host)
        return True

    def fab_task_bt(self, args):
        cmd_file = args.strip()[2:].strip()
        print(cmd_file)
        try:
            with open(cmd_file, 'r') as f:
                for line in f.readlines():
                    self.fab_run(line.strip())
                return True
        except Exception as e:
            ColorPrint.red(e)
        return True

    @need_confirm
    def fab_task_rm(self, args):
        """
        rm command
        :param args:
        :return:
        """
        self.fab_run(args)
        return True


    def do_fab(self, args):
        """
        -*-*-*-*-*-*-*-*-
        desc : execute shell command on remote hosts
        usage: fab [-fabp/-fabs] [-fabh host1,host2,...] cmd [&]
               & : running task on deamon
               -fabp : parallel executing command on all hosts
               -fabs : serial executing command on all hosts
               -fabh host1,host2,...: execute command on just host1,host2...
        eg   : fab -fabh [host1,host2] php test.php &
               fab -fabp mkdir /home/work/hello
        """
        try:
            args_list = args.split()
            fab_kw = {}
            for arg in args.split():
                if arg.startswith('-fab'):
                    self.callback('set_', arg[1:], args_list, fab_kw)
            
            args = ' '.join(args_list)
            # check whether the task is running on deamon
            if args[-1] == '&':
                args = 'nohup ' + args + ' disown;sleep 1'
            with fab.settings(**fab_kw):
                # 需要定制开发的命令
                if self.callback('fab_task_', args_list[0], args):
                    return
                self.fab_run(args)

        except Exception as e:
            ColorPrint.red('cmd error, %s' % e)
            ColorPrint.blue(self.do_fab.__doc__)

    def do_setparallel(self, args):
        """
        -*-*-*-*-*-*-*-*-
        desc : set execute parallel status (default False)
        usage: setparallel [True/true/False/false]
        """
        if args in ('True', 'true', 'False', 'false'):
            fab.env.parallel = args.capitalize()
            print('set execute parallel %s successfully!' % args)

    @fab_execute
    def fab_put(self, local_path, remote_path):
        fab.put(local_path, remote_path)

    def do_put(self, args):
        """
        -*-*-*-*-*-*-*-*-
        desc : put local files to remote servers
        usage: put local_path remote_path
        eg   : put ./hosts.ini ~/test/
        """
        args_list = args.split()
        if len(args_list) < 2:
            ColorPrint.blue(self.do_put.__doc__)
        else:
            self.fab_put(args_list[0], args_list[1])

    @fab_execute
    def fab_get(self, remote_path, local_path):
        fab.get(remote_path, local_path)

    def do_get(self, args):
        """
        -*-*-*-*-*-*-*-*-
        desc : get files from remote servers
        usage: get [-n] local_path remote_path
               -n : save remote file to file0, file1...
               default : filehostname0, filehostname1...
        eg   : get -n /home/work/eg ~/get/test.txt
               /home/work/eg/test.txt0
               /home/work/eg/test.txt1
               ...
               get /home/work/eg ~/get/test.txt
               /home/work/eg/test.txtnj02-vs-ting-ensl-34.nj02
               /home/work/eg/test.txtcq02-lightapp-stat09.cq02
        """
        try:      
            args_list = args.split()
            if '-n' in args_list:
                args_list.remove('-n')
           
            local_path = args_list[0]
            remote_path = args_list[1]
            if len(fab.env.hosts) > 1:
                prefix = remote_path.split('/')[-1]
                for i, host in enumerate(fab.env.hosts):
                    with fab.settings(hosts=[host]):
                        tmp_local_path = local_path+'/'+prefix+host
                        self.fab_get(remote_path, tmp_local_path)
            else:
                self.fab_get(remote_path, local_path)
        except IndexError:
            ColorPrint.red('not enough parameters')
        except Exception as e:
            ColorPrint.red('bad command, %s' % e)
            ColorPrint.blue(self.do_get.__doc__)

    def do_shell(self, args):
        """
        -*-*-*-*-*-*-*-*-
        desc : run a shell commad on local server
        usage: !shellcmd
        eg   : !ls
        """
        sub_cmd = subprocess.Popen(args, shell=True, stdout=subprocess.PIPE)
        print(sub_cmd.communicate()[0])

    def callback(self, prefix, name, *args):
        method = getattr(self, prefix+name, None)
        if callable(method):
            return method(*args)
            # method(*args)

    def ls_g(self):
        for group in self.host_groups.keys():
            print(group)
        print()

    def ls_a(self):
        for group, hosts in self.host_groups.items():
            print(blue(group+':'))
            for host in hosts:
                print(host)
            print()
        print()

    def ls_env(self):
        print('user :', fab.env.user)
        print('cwd  :', fab.env.cwd)
        print('parallel :', fab.env.parallel)
        print()

    def do_ls(self, args):
        """
        -*-*-*-*-*-*-*-*-
        desc : list remote hosts
        usage: list [-a/-g/-env]
               list : list current hosts
               -g : list all group
               -a : list all hosts
               -env : list environment variables
        """
        args = args.strip()
        if not args.strip():
            for i, host in enumerate(fab.env.hosts):
                print('%d | %s' % (i, host))
        else:
            name = args[1:]
            self.callback('ls_', name)


    def do_addgrp(self, args):
        """
        -*-*-*-*-*-*-*-*-
        desc : add a new remote host group,
        usage: addgrp group_name host1 host2 ... hostn
        """
        args_list = args.split()
        try:
            group_name = args_list[0]
            hosts = args_list[1:]
        except IndexError:
            ColorPrint.blue(self.do_addgrp.__doc__)
        else:
            self.host_groups[group_name] = hosts
            self.conf_changed = True

    def do_rmgrp(self, args):
        '''
        -*-*-*-*-*-*-*-*-
        desc : remove a remote host group,
        usage: rmgrp group_name
        '''
        group = args.strip()
        if group:
            try:
                self.host_groups.pop(group)
            except KeyError:
                ColorPrint.red('not find %s host group' % group)
                print()
            else:
                self.conf_changed = True

    def set_cmd_prompt(self, meg):
        self.prompt = yellow("%s%s " % (meg, CMD_PROMPT))

    def do_at(self, args):
        """
        -*-*-*-*-*-*-*-*-
        desc : switch to a group or a host
        usage: at group_name/host_name
        """
        args = args.strip()
        switch = False
        for group, hosts in self.host_groups.items():
            if args == group:
                fab.env.hosts = hosts
                self.set_cmd_prompt(args)
                switch = True
                break
            if args in hosts:
                fab.env.hosts = [args, ]
                self.set_cmd_prompt(args)
                switch = True
                break
        if not switch:
            ColorPrint.red('Error : can not find any group or host')

    def do_pwd(self, args):
        """
        -*-*-*-*-*-*-*-*-
        desc : show the current directory path
        usage: pwd
        """
        print(fab.env.cwd if fab.env.cwd else '/home/work')
        print()

    def do_exit(self, args):
        """
        desc : terminatesthe application
        usage: exit
        """
        # disconnect_all()
        if self.conf_changed:
            self.save_config(self.conf_file)
        return True

    def emptyline(self):
        """
        when input nothing cmd module will execute last command.
        redefine emptyline function
        :return:
        """
        pass



#!/usr/bin/env python
import codecs
import os.path
import re
import sys

import shutil
from setuptools import setup, find_packages


here = os.path.abspath(os.path.dirname(__file__))


def read(*parts):
    return codecs.open(os.path.join(here, *parts), 'r').read()


def find_version(*file_paths):
    version_file = read(*file_paths)
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


requires = [
            'configparser>=3.5.0',
            'six',
            'fabric==1.14.0',
            'setuptools',
            'cryptography>=2.4.2'
]
if sys.platform == 'win32':
    requires.append('pywin32')

if sys.version_info[:2] == (2, 6):
    # For python2.6 we have to require argparse since it
    # was not in stdlib until 2.7.
    requires.append('argparse>=1.1')


setup_options = dict(
    name='fablinker',
    version=find_version("fablinker", "constant.py"),
    description='A tool for operating servers.',
    long_description=open('README.rst').read(),
    author='GaryLi',
    author_email='gangly123@163.com',
    url='https://github.com/gangly/fablinker',
    scripts=['bin/fablinker',
             ],
    packages=find_packages(exclude=['tests*']),
    package_data={'fablinker': ['data/*.json', 'data/*.ini', 'examples/*/*.rst',
                             'examples/*/*/*.rst']},
    install_requires=requires,
    extras_require={
        ':python_version=="2.6"': [
            'argparse>=1.1',
        ]
    },
    license="Apache License 2.0",
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Natural Language :: English',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
)


def get_full_path(path):
    fullpath = os.path.expandvars(path)
    fullpath = os.path.expanduser(fullpath)
    fullpath = os.path.abspath(fullpath)
    return fullpath

FABLINKER_DIR = get_full_path(os.path.join('~', '.fablinker'))
CONFIG_FILE = os.path.join(FABLINKER_DIR, 'fabconf.ini')
# DEFAULT_CONFIG_FILE = os.path.join(FABLINKER_DIR, 'default.ini')

ROOT_DIR = os.path.dirname(__file__)
HCMD_INI = os.path.join(ROOT_DIR, 'fablinker', 'data', 'fabconf.ini')


if not os.path.exists(FABLINKER_DIR):
    os.makedirs(FABLINKER_DIR)
# shutil.copyfile(HCMD_INI, DEFAULT_CONFIG_FILE)
if not os.path.exists(CONFIG_FILE):
    shutil.copyfile(HCMD_INI, CONFIG_FILE)

if 'py2exe' in sys.argv:
    # This will actually give us a py2exe command.
    import py2exe
    # And we have some py2exe specific options.
    setup_options['options'] = {
        'py2exe': {
            'optimize': 0,
            'skip_archive': True,
            'dll_excludes': ['crypt32.dll'],
            'packages': ['urllib', 'httplib', 'HTMLParser',
                         'fablinker', 'ConfigParser', 'xml.etree', 'pipes'],
        }
    }
    setup_options['console'] = ['bin/fablinker']


BIN_HCMD = '/usr/bin/fablinker'
LOCAL_HCMD = '/usr/local/bin/fablinker'
if sys.platform != 'win32' and os.path.exists(BIN_HCMD):
    os.remove(BIN_HCMD)
if sys.platform != 'win32' and os.path.exists(LOCAL_HCMD):
    os.remove(LOCAL_HCMD)

setup(**setup_options)

if sys.platform != 'win32' and os.path.exists(BIN_HCMD) and not os.path.exists(LOCAL_HCMD):
    os.symlink(BIN_HCMD, LOCAL_HCMD)